--- pgiosim.c.orig	2009-09-28 07:41:16.000000000 +0200
+++ pgiosim.c	2009-09-30 09:01:11.000000000 +0200
@@ -25,6 +25,10 @@
 #define TV2DOUBLE(tv) ((double)(tv)->tv_sec + (double)((tv)->tv_usec) / 1000000)
 int stop = 0;
 
+#define STANDARD_BLOCK_SIZE 8192
+
+unsigned int BLOCK_SIZE = (STANDARD_BLOCK_SIZE);
+
 typedef struct _FileInfo
 {
   int fd;
@@ -62,7 +66,7 @@
   f = malloc(sizeof(FileInfo));
   stat(fn, &sinfo);
 
-  f->numBlocks = sinfo.st_size / 8192;
+  f->numBlocks = sinfo.st_size / (BLOCK_SIZE);
   f->fd = open(fn, flags);
   f->curBlock = 0;
   if(f->fd <= 0)
@@ -77,7 +81,7 @@
 int main(int argc, char **argv)
 {
   int count;
-  char buf[8193];
+  char* buf;
   FileInfo *files[25]; // up to 25 files we can use. 
   int numFiles;
   int x;
@@ -89,6 +93,9 @@
   int writePercent = 0;
   int writeBlocks = 0;
   int fsyncAfterWrite = 0;
+  int singlereadrequests = 0;
+
+  double blocksize_factor = 1.0;
 
   char ch;
   FileInfo *f;
@@ -99,7 +106,7 @@
 
   signal(SIGINT, doSIGINT);
 
-  while((ch = getopt(argc, argv, "yb:sw:")) != EOF)
+  while((ch = getopt(argc, argv, "yb:sw:o:r")) != EOF)
 	{
 	  switch(ch)
 		{
@@ -122,10 +129,17 @@
 			}
 		  printf("Write Mode: %d%%\n", writePercent);
 		  break;
-		  
+		case 'o':
+		  BLOCK_SIZE = atoi(optarg);
+		  break;
+		case 'r':
+		  singlereadrequests = 1;
+		  break;
 		}
 	}
 
+  buf = malloc(BLOCK_SIZE + 1);
+
   argv += optind;
   argc -= optind;
 
@@ -136,6 +150,16 @@
 	  files[numFiles] = getFile(argv[numFiles], writeMode);
 	}
 
+  printf("blocksize=%u, ", BLOCK_SIZE);
+  if (singlereadrequests)
+  {
+    printf("doing single read requests with chunk size of %u bytes\n", STANDARD_BLOCK_SIZE);
+  }
+  else
+  {
+    printf("reading block as a whole\n");
+  }
+
   srand(getpid());
 
   gettimeofday(&tv, NULL);
@@ -163,29 +187,60 @@
 		  printf("Dork\n");
 		}
 
-	  if(lseek(f->fd, (off_t)8192 * cb, SEEK_SET) == -1)
+	  if(lseek(f->fd, (off_t)(BLOCK_SIZE) * cb, SEEK_SET) == -1)
 		{
-		  printf("Seek %d failed: %d %s\n", x, cb * 8192, strerror(errno));
+	  	  printf("Seek %d failed: %d %s\n", x, cb * (BLOCK_SIZE), strerror(errno));
 		}
 
-	  n = read(f->fd, buf, 8192);
+	  if (!singlereadrequests)
+	  {
+		n = read(f->fd, buf, (BLOCK_SIZE));
+
+	  	//	  printf("N: %d\n", n);
+	  	if(n != (BLOCK_SIZE))
+			{
+		  	  printf("Bad read: %s\n", strerror(errno));
+			}
+	  }
+	  else
+	  {
+		int blocks_left = BLOCK_SIZE;
 
-	  //	  printf("N: %d\n", n);
-	  if(n != 8192)
+		while (blocks_left > 0)
 		{
-		  printf("Bad read: %s\n", strerror(errno));
+			int current_block_size = STANDARD_BLOCK_SIZE;
+
+			if (blocks_left >= STANDARD_BLOCK_SIZE)
+			{
+				blocks_left -= STANDARD_BLOCK_SIZE;
+			}
+			else
+			{
+				current_block_size = blocks_left;
+				blocks_left = 0;
+			}
+
+			n = read(f->fd, buf, current_block_size);
+
+		  	//	  printf("N: %d\n", n);
+	 	 	if(n != current_block_size)
+			{
+		  	  printf("Bad read: %s\n", strerror(errno));
+			}
+
 		}
+	  }
 
 	  if(writeMode)
 		{
 		  // we just write the block back.
 		  if(rand() % 100 < writePercent)
 			{
-			  if(lseek(f->fd, (off_t)8192 * cb, SEEK_SET) == -1)
+			  if(lseek(f->fd, (off_t)(BLOCK_SIZE) * cb, SEEK_SET) == -1)
 				{
-				  printf("Seek %d failed: %d %s\n", x, cb * 8192, strerror(errno));
+				  printf("Seek %d failed: %d %s\n", x, cb * (BLOCK_SIZE), strerror(errno));
 				}
-			  if(write(f->fd, buf, 8192) != 8192)
+			  if(write(f->fd, buf, (BLOCK_SIZE)) != (BLOCK_SIZE))
 				{
 				  printf("Write Failed: %s\n", strerror(errno));
 				  exit(0);
@@ -218,8 +273,13 @@
   elapsed = end - start;
   printf("Elapsed: %0.2f\n", elapsed);
   printf("Read %d blocks Wrote %d blocks\n", numReads, writeBlocks);
-  printf("%0.2f op/sec, %0.2fkB/sec\n", (double)numReads / elapsed, 
-		 (((double)numReads * 8192) / elapsed) / 1024);
+  
+  if (singlereadrequests)
+     blocksize_factor = (double)BLOCK_SIZE / STANDARD_BLOCK_SIZE;
+
+  printf("%0.2f op/sec, %0.2fkB/sec\n", (double)numReads * blocksize_factor / elapsed, 
+		 (((double)numReads * (BLOCK_SIZE)) / elapsed) / 1024);
 
+  free(buf);
 }
 
